from .baseline import *
from .filters import *
from .functions import *
from .ml_regressor import *
from .tlcorrection import *
from .spectranization import *
from .rameau import *
from .mixing import *



# -*- coding: utf-8 -*-
import numpy as np
from scipy import interpolate
from scipy import signal
from scipy.optimize import curve_fit
from scipy.interpolate import UnivariateSpline
from scipy.interpolate import interp1d

import rampy

# SPECIFIC FUNCTIONS FOR TREATMENT OF SPECTRA

def spectrarray(name,sh,sf,x):
    """Construct a general array that contain common X values in first columns and all Y values in the subsequent columns.

    Parameters
    ----------
    name : ndarray
        Array containing the names of the files (should work with a dataframe too).
    sh : int
        Number of header line in files to skip.
    sf : int
        Number of footer lines in files to skip.
    x : ndarray
        The common x axis.

    Returns
    -------
    out
        An array with the common X axis in first column and all the spectra in the subsequent columns.
    """
    for i in range(len(name)):
        rawspectre = np.genfromtxt(name[i],skip_header=sh, skip_footer=sf)
        rawspectre = rawspectre[~np.isnan(rawspectre).any(1)] # check for nan

        y = resample(rawspectre[:,0],rawspectre[:,1],x) # resample the signal

        # Now we construct the output matrix
        # 1st column is the x axis
        # then others are the spectra in the order provided in the list of names input array
        if i == 0:
            out = np.zeros((len(x),len(name)+1))
            out[:,0]=x
            out[:,i+1]=y
        else:
            out[:,i+1]=y

    return out

def spectrataux(spectres):
    """Calculate the increase/decrease rate of each frequencies in a set of spectra.

    Parameters
    ----------
    spectres : ndarray
        An array of spectra containing the common X axis in first column and all the spectra in the subsequent columns. (see spectrarray function)

    Returns
    -------
    taux : ndarray
        The rate of change of each frequency, fitted by a 2nd order polynomial functions.
    """
    # we need an organized function before calling the curve_fit algorithm
    freq = spectres[:,0]
    # output array
    taux = np.zeros((len(freq),4));
    taux[:,0] = freq[:]

    # We look a each frequency, we sort y data and fit them with a second order polynomial
    for i in range(len(freq)):
        y = spectres[i,1::]
        popt, pcov = curve_fit(fun2,x,y,[0.5e-3,0.5e-4,1e-6])
        taux[i,1:len(x)]=popt

    return taux

def spectraoffset(spectre,oft):
    """Offset your spectra with values in offsets

    Parameters
    ----------
    spectre : ndarray
        array of spectra constructed with the spectrarray function
    oft : ndarray
        array constructed with numpy and containing the coefficient for the offset to apply to spectra

    Returns
    -------
    out : ndarray
        Array with spectra separated by offsets defined in oft

    """

    out = np.zeros(spectre.shape) # we already say what is the output array
    for i in range(len(oft)): # and we offset the spectra
        out[:,i+1] = spectre[:,i+1] + oft[i]
    return out

#
# Simple data treatment functions
#
def flipsp(sp):
    """Flip an array along the row dimension (dim = 1) if the row values are in decreasing order.

    Parameters
    ----------
    sp : ndarray
        An array with n columns
    Returns
    -------
    sp : ndarray
        The same array but sorted such that the values in the first column are in increasing order.
    """
    if sp[-1,0] < sp[0,0]:
        sp = np.flip(sp,0)
        return sp
    else:
        return sp

def resample(x,y,x_new,**kwargs):
    """Resample a y signal associated with x, along the x_new values.

    Parameters
    ----------
    x : ndarray
        The x values
    y : ndarray
        The y values
    x_new : ndarray
        The new X values

    Options
    -------
    Arguments to be passed to scipy.interpolate.interp1d, see https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.interp1d.html

    kind : str or int, optional
        Specifies the kind of interpolation as a string (‘linear’, ‘nearest’, ‘zero’, ‘slinear’, ‘quadratic’, ‘cubic’, ‘previous’, ‘next’, where ‘zero’, ‘slinear’, ‘quadratic’ and ‘cubic’ refer to a spline interpolation of zeroth, first, second or third order; ‘previous’ and ‘next’ simply return the previous or next value of the point) or as an integer specifying the order of the spline interpolator to use. Default is ‘linear’.
    axis : int, optional
        Specifies the axis of y along which to interpolate. Interpolation defaults to the last axis of y.
    copy : bool, optional
        If True, the class makes internal copies of x and y. If False, references to x and y are used. The default is to copy.
    bounds_error : bool, optional
        If True, a ValueError is raised any time interpolation is attempted on a value outside of the range of x (where extrapolation is necessary). If False, out of bounds values are assigned fill_value. By default, an error is raised unless fill_value=”extrapolate”.
    fill_value : array-like or (array-like, array_like) or “extrapolate”, optional
        if a ndarray (or float), this value will be used to fill in for requested points outside of the data range. If not provided, then the default is NaN. The array-like must broadcast properly to the dimensions of the non-interpolation axes.
        If a two-element tuple, then the first element is used as a fill value for x_new < x[0] and the second element is used for x_new > x[-1]. Anything that is not a 2-element tuple (e.g., list or ndarray, regardless of shape) is taken to be a single array-like argument meant to be used for both bounds as below, above = fill_value, fill_value.

        New in scipy version 0.17.0.

        If “extrapolate”, then points outside the data range will be extrapolated.

        New in scipy version 0.17.0.

    assume_sorted : bool, optional
        If False, values of x can be in any order and they are sorted first. If True, x has to be an array of monotonically increasing values.

    Returns
    -------
    y_new : ndarray
        y values interpolated at x_new.

    Remarks
    -------
    Uses scipy.interpolate.interp1d
    """
    f = interp1d(x,y,**kwargs)
    return f(x_new)

def normalise(y,x=0,method="intensity"):
    """normalisation of the y signal
    Parameters
    ==========
    x : ndarray
        x values
    y : ndarray
        y values
    method : string
        method used, choose between area, intensity, minmax
    Returns
    =======
    y_norm : Numpy array
        Normalised signal
    """
    if method == "area":
        if x == 0:
            raise TypeError("Input x values for area normalisation")
        y = y/np.trapz(y,x)
    if method == "intensity":
        y = y/np.max(y)
    if method == "minmax":
        y = (y-np.min(y))/(np.max(y)-np.min(y))

def centroid(x,y,smoothing=False,**kwargs):
    """calculation of the y signal centroid
    
    as np.sum(y/np.sum(y)*x)
    if smoothing == 1:
    
    Parameters
    ==========
    x: Numpy array
        x values
    y: Numpy array
        y values, 1 spectrum

    Options
    =======
    smoothing : bool
        True or False. Smooth the signals with arguments provided as kwargs. Default method is whittaker smoothing. See the rampy.smooth function for smoothing options and arguments.
        
    Returns
    =======
    centroid : float
        signal centroid
    """
    
    # for safety, we reshape the x array    
    x = x.reshape(-1)
    y = y.reshape(-1)
    
    if smoothing == True:
        y_ = rampy.smooth(x,y,**kwargs)
    else: 
        y_ = y.copy()
        
    return np.sum(y_/np.sum(y_)*x)


# -*- coding: utf-8 -*-
#############################################################################
#Copyright (c) 2018 Charles Le Losq
#
# Licence GNU-GPL
#
#
#############################################################################
import numpy as np
from scipy.constants import c, h, k

def tlcorrection(x,y,temp,wave, **kwargs):
    """
        tlcorrection(x,y,temp,wave,**kwargs)

    Parameters
    ----------
    x
        Raman shifts in cm-1
    y
        Intensity values as counts
    temp
        Temperature in °C
    wave
        wavenumber of the laser that excited the sample, in nm

    kwargs
    ------
    correction
        Equation used for the correction. Choose between 'long', 'galeener', or 'hehlen'. Default = 'long'.
    normalisation
        Data normalisation procedure. Choose between 'intensity', 'area', or 'no'. Default = 'area'.
    density
        The density of the studied material in kg m-3, to be used with the 'hehlen' equation. Default = 2210.0 (density of silica).

    Returns
    -------
    x
        The Raman shifts values.
    long
        The corrected intensities.
    eselong
        The errors calculated as sqrt(y) on raw intensities and propagated after the correction.

    Remarks
    -------

    This correction uses the formula reported in Galeener and Sen (1978), Mysen et al. (1982), Brooker et al. (1988) and Hehlen et al. (2010).

    The 'galeener' equation is the exact one reported in Galeener and Sen (1978), which is a modification from Shuker and Gammon (1970) for accounting of (vo - v)^4 dependence of the Raman intensity. See also Brooker et al. (1988) for further discussion.

    The 'long' equation is that of Galeener and Sen (1978) corrected by a vo^3 coefficient for removing the cubic meter dimension of the equation of 'galeener'. This equation has been used in Mysen et al. (1982), Neuville and Mysen (1996) and Le Losq et al. (2012).

    The 'hehlen' equation is that reported in Hehlen et al. (2010). It actually originates before this publication (Brooker et al. 1988). It uses a different correction that avoid crushing the signal below 500 cm-1. THerefore, it has the advantage of keeping intact the Boson peak signal in glasses.
    """

    if x[-1] < x[0]: # to raise an error if decreasing x values are provided
        raise ValueError('x values should be increasing.')

    # getting the kwargs
    correction = kwargs.get('correction','long')
    normalisation = kwargs.get('normalisation','area')
    density = kwargs.get('density',2210.0)

    #h = 6.626070040e-34   # J S    Plank constant from NIST
    #hb = 1.054571800e-34 # J S    Reduced Plank constant from NIST
    #k = 1.38064852e-23      # J K-1    Boltzman constant from NIST
    #c = 299792458.         # M S-1    Speed of light from NIST
    nu0 = 1.0/wave*1e9     # nu0 laser is in M-1 (wave is in nm)
    T = temp + 273.15    # K temperature
    # density is in KG M-3

    # Calculate the relative error on data as sqrt(y). If y <= 0, then error = abs(y).
    ese = np.sqrt(np.absolute(y))/np.absolute(y) # relative errors

    # get the Raman shift in m-1
    nu = 100.0*x # cm-1 -> m-1 Raman shift


    # then we proceed to the correction
    try:
        if correction == 'long':
            # Formula used in Mysen et al. (1982), Neuville and Mysen (1996) and Le Losq et al. (2012) (corrected for using the Planck constant in the last reference)
            # It is that reported in Brooker et al. (1988) with the addition of a scaling nu0^3 coefficient for adimentionality
            frequency = nu0**3*nu/((nu0-nu)**4) # frequency correction; dimensionless
            boltzman = 1.0 - np.exp(-h*c*nu/(k*T)) # temperature correction with Boltzman distribution; dimensionless
            ycorr = y*frequency*boltzman # correction

        elif correction == 'galeener':
            # This uses the formula reported in Galeener and Sen (1978) and Brooker et al. (1988); it uses the Bose-Einstein / Boltzman distribution
            # Formula from  without the scaling vo^3 coefficient reported in Mysen et al. (1982), Neuville and Mysen (1996) and Le Losq et al. (2012)
            frequency = nu/((nu0-nu)**4) # frequency correction; M^3
            boltzman = 1.0 - np.exp(-h*c*nu/(k*T)) # temperature correction with Boltzman distribution; dimensionless
            ycorr = y*frequency*boltzman # correction

        elif correction =='hehlen':
            # this uses the formula reported in Hehlen et al. 2010
            frequency = 1/(nu0**3*density) # frequency + density correction; M/KG
            boltzman = 1.0 - np.exp(-h*c*nu/(k*T)) # dimensionless
            ycorr = nu*y*frequency*boltzman # correction
    except:
        print("Not implemented, choose correction = long, galeener or hehlen.")

    # we take care of the normalisation
    try:
        if normalisation == 'area':
            ycorr = ycorr/np.trapz(ycorr,x) # area normalisation
        elif normalisation == 'intensity':
            ycorr = ycorr/np.max(ycorr) # max. intensity normalisation
        elif normalisation == 'no':
            print("No normalisation...")
    except:
        print("Set the optional normalisation parameter to area, intensity or no.")

    esecorr = ese*ycorr # error calculation

    return x, ycorr, esecorr



import cvxpy
import numpy as np

def mixing_sp(y_fit,ref1,ref2):
    """mix two reference spectra to match the given ones

    Parameters
    ----------
    y_fit : ndarray, shape m * n
        an array containing the signals with m datapoints and n experiments
    ref1 : ndarray, shape m
        an array containing the first reference signal
    ref2 : ndarray, shape m
        an array containing the second reference signal

    Returns
    -------
    out : ndarray, shape n
        the fractions of ref1 in the mix

    Notes
    -----
    Performs the calculation by minimizing the sum of the least absolute value of the objective function:
        obj = sum(abs(y_fit-(ref1*F1 + ref2*(1-F1))))

    Uses cvxpy to perform this calculation
    """

    ref1 = ref1.reshape(1,-1)
    ref2 = ref2.reshape(1,-1)
    
    F1 = cvxpy.Variable(shape=(y_fit.shape[1],1))

    objective = cvxpy.Minimize(cvxpy.sum(cvxpy.abs(F1*ref1 + (1-F1)*ref2 - y_fit.T))) 

    constraints = [0 <= F1, F1 <= 1]

    prob = cvxpy.Problem(objective, constraints)
    prob.solve()
    return np.asarray(F1.value).reshape(-1)



#!/usr/bin/env python
#-*- coding: utf-8 -*-
import numpy as np
import scipy
from scipy import signal
import scipy.sparse as sparse

def smooth(x,y,method="whittaker",**kwargs):
    """smooth the provided y signal (sampled on x)

    Parameters
    ==========
    x: ndarray
        Nx1 array of x values (equally spaced).
    y: ndarray
        Nx1 array of y values (equally spaced).
    method: str
        Method for smoothing the signal;
        choose between savgol (Savitzky-Golay), GCVSmoothedNSpline, MSESmoothedNSpline, DOFSmoothedNSpline, whittaker, 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'.

    kwargs
    ======
    window_length: int
        The length of the filter window (i.e. the number of coefficients). window_length must be a positive odd integer.
    polyorder: int
        The order of the polynomial used to fit the samples. polyorder must be less than window_length.
    Lambda: float
        smoothing parameter of the Whittaker filter described in Eilers (2003). The higher the smoother the fit.
    d: int
        d parameter in Whittaker filter, see Eilers (2003).
    ese_y: ndarray
        errors associated with y (for the gcvspline algorithms)

    Returns
    =======
    y_smo: ndarray
        smoothed signal sampled on x.

    Note
    ====

    Use of GCVSmoothedNSpline, MSESmoothedNSpline, DOFSmoothedNSpline requires installation of gcvspline. See gcvspline documentation.
    See also documentation for details on GCVSmoothedNSpline, MSESmoothedNSpline, DOFSmoothedNSpline.

    savgol uses the scipy.signal.savgol_filter() function.

    References
    ==========
    Eilers, P.H.C., 2003. A Perfect Smoother. Anal. Chem. 75, 3631–3636. https://doi.org/10.1021/ac034173t

    Scipy Cookbook: https://scipy-cookbook.readthedocs.io/items/SignalSmooth.html?highlight=smooth

    """
    window_len = kwargs.get("window_length",5)
    polyorder = kwargs.get("polyorder",2)
    lam = kwargs.get("Lambda",10.0**5)
    d = kwargs.get("d",2)
    ese_y = kwargs.get("ese_y",1.0)

    if x.size < window_len:
        raise ValueError("Input vector needs to be bigger than window size.")

    if not method in ['GCVSmoothedNSpline','MSESmoothedNSpline','DOFSmoothedNSpline','whittaker','savgol',
                      'flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError("Method should be on 'GCVSmoothedNSpline','MSESmoothedNSpline','DOFSmoothedNSpline','whittaker','savgol','flat', 'hanning', 'hamming', 'bartlett', 'blackman'")

    if (method ==  "GCVSmoothedNSpline") or (method == "MSESmoothedNSpline") or (method == "DOFSmoothedNSpline"): # gcvspline methods
        
        try: # we test if gcvspline is installed
            import gcvspline
        except ImportError:
            print('ERROR: Install gcvspline to use this smoothing mode (needs a working FORTRAN compiler).')

        w = 1.0 / (np.ones((y.shape[0],1)) * ese_y) # errors

        if method == "GCVSmoothedNSpline":
            flt = gcvspline.GCVSmoothedNSpline(x.reshape(-1),y.reshape(-1),w.reshape(-1))
        elif method == "MSESmoothedNSpline":
            flt = gcvspline.MSESmoothedNSpline(x.reshape(-1),y.reshape(-1),w.reshape(-1))
        elif method == "DOFSmoothedNSpline":
            flt = gcvspline.DOFSmoothedNSpline(x.reshape(-1),y.reshape(-1),w.reshape(-1))
        return flt(x)

    elif method == "whittaker": # whittaker smoother
        return whittaker(y,Lambda=lam,d=d)
    elif method == "savgol": # Savtisky-Golay filter
        return scipy.signal.savgol_filter(y, window_len, polyorder)
    elif method in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']: # various window filters, from https://scipy-cookbook.readthedocs.io/items/SignalSmooth.html?highlight=smooth

        s=np.r_[y[window_len-1:0:-1],y,y[-2:-window_len-1:-1]]
        if method == 'flat': #moving average
            w=np.ones(window_len,'d')
        else:
            w=eval('np.'+method+'(window_len)')

        y_filt = np.convolve(w/w.sum(),s,mode='valid')
        shift = int((len(y_filt) - len(y))/2)

        return y_filt[shift:-shift]

def whittaker(y,**kwargs):
    """smooth a signal with the Whittaker smoother

    Inputs
    ------
    y: ndarray
        An array with the values to smooth (equally spaced).

    kwargs
    ------
    Lambda: float
        The smoothing coefficient, the higher the smoother. Default = 10^5.

    Outputs
    -------
    z: ndarray
        An array containing the smoothed values.

    References
    ----------
    P. H. C. Eilers, A Perfect Smoother. Anal. Chem. 75, 3631–3636 (2003).

    """
    # optional parameters
    lam = kwargs.get('Lambda',1.0*10**5)

    # starting the algorithm
    L = len(y)
    D = scipy.sparse.csc_matrix(np.diff(np.eye(L), 2))
    w = np.ones(L)
    W = scipy.sparse.spdiags(w, 0, L, L)
    Z = W + lam * D.dot(D.transpose())
    z = scipy.sparse.linalg.spsolve(Z, w*y)

    return z

def spectrafilter(spectre,filtertype,fq,numtaps,columns):
    """Filter specific frequencies in spectra with a butterworth filter

    Inputs
    ------
    spectre
        Array of X-Y values of spectra. First column is X and subsequent n columns are Y values of n spectra. (see also spectraarray function)
    filtertype
        Contains a string defining which type of filter you want. Choose between 'low', 'high', 'bandstop', 'bandpass'.
    fq
        Frequency of the periodic signal you try to erase. If using a bandpass or band stop filter, fq must be an array containing the cutoff frequencies.
    columns
        An array defining which columns you want to treat.

    Outputs
    -------
    out
        An array with the filtered signals.

    """

    # we already say what is the output array
    out = np.zeros(spectre.shape)

    # Butterworth band stop filter caracteristics
    a = spectre[1,0] - spectre[0,0]
    samplerate = 1/a  #Hertz
    nyq_rate = samplerate/2 # frequence Nyquist
    cutf = fq # cutoff frequency
    #bandwidth = 0.005 # largeur filtre, for band pass/stop filters
    numtaps = 1 # ordre du filtre...

    for i in range(len(columns)):
        y = spectre[:,columns[i]]
        if (filtertype == 'low') or (filtertype == 'high'):
            b, a = signal.butter(numtaps, [(cutf/nyq_rate)], btype = filtertype)
            out[:,columns[i]] = signal.filtfilt(b, a, y) # filter with phase shift correction
        else:
            b, a = signal.butter(numtaps, [(cutf[0]/nyq_rate),(cutf[1]/nyq_rate)], btype = filtertype)
            out[:,columns[i]] = signal.filtfilt(b, a, y) # filter with phase shift correction

    # Note forgetting to register the x axis
    out[:,0] = spectre[:,0]

    return out



# -*- coding: utf-8 -*-
#############################################################################
#Copyright (c) 2018 Charles Le Losq
#
# Licence GNU-GPL
#
#
#############################################################################

import numpy as np
import pandas as pd
import scipy
import rampy as rp

class rameau:
    """treat Raman spectra of glass to retrieve the glass water content

    Parameters
    ==========
    data_liste: Pandas dataframe
        A Pandas dataframe containing the data and various meta information.

    Attributes
    ==========
    x: ndarray
        a 1D array (Nx1) containing the common x axis (wavelength) of the spectra.
    y: ndarray
        a NxM array (with M the number of spectra) containing the raw intensities of the spectra.
    y_corr: ndarray
        a NxM array (with M the number of spectra) containing the corrected intensities of the spectra.
    y_base: ndarray
        a NxM array (with M the number of spectra) containing the backgrounds of the spectra.
    rws: ndarray
        a 1D array (Nx1) containing the ratio between the integrated intensities of the water and silicate signals.
    rw: ndarray
        a 1D array (Nx1) containing the integrated intensities of the water signal.
    rs:
        a 1D array (Nx1) containing the integrated intensities of the silicate signals.
    water:
        the known glass water content provided in data_liste (set to 0 if predicting for unknowns)
    water_predicted:
        the predicted glass water content provided in data_liste (set to 0 if predicting for unknowns)
    p: ndarray
        calibration coefficient(s) of the LL2012 or DG2017 method
    names: pandas dataframe
        filenames indicated in the data_liste input 

    Note
    ====
    Uses either the LL2012 method (Le Losq et al., 2012) or the DG2017 (Di Genova et al., 2017) method. See references.

    In the LL2012 method, a cubic spline is fitted to the regions of interest provided in self.data_liste (see example).
    The spline is smoothed by the spline_coeff of the data_reduction method. The water content is calculated following eq. (3) of LL2012, with the A coefficient either provided or calculated by the method self.calibrate().

    In the DG2017 method, a third-order polynomial is fitted to the spectra following the instructions of Di Genova et al. (2017).
    The water content is calculated as wt% H2O = Rws * (a * [FeO wt%] + b) with a and b the coefficients either provided or calculated by the method self.calibrate().

    References
    ==========
    LL2102: C. Le Losq, D. R. Neuville, R. Moretti, J. Roux, Determination of water content in silicate glasses using Raman spectrometry: Implications for the study of explosive volcanism. American Mineralogist. 97, 779–790 (2012).
    DG 2017 D. Di Genova et al., Effect of iron and nanolites on Raman spectra of volcanic glasses: A reassessment of existing strategies to estimate the water content. Chemical Geology. 475, 76–86 (2017).


    """
    # class variables
    x = []
    y = []
    y_corr = []
    y_base = []
    rws = []
    rw = []
    rs = []
    water = []
    water_predicted = []
    p = []
    names = []

    def __init__(self,data_liste):
        self.data_liste = data_liste
        self.water = np.asarray(data_liste["Water, wt%"])
        self.names = data_liste["Name"]

        try: # we test if gcvspline is installed
            import gcvspline
        except ImportError:
            print('gcvspline module not found. Please install it and check it works (requires a FORTRAN compiler).')

    def data_reduction(self,method="LL2012",delim='\t',path_in='./raw/',laser=514.532,spline_coeff=0.001,poly_coeff=3):
        """process Raman spectra of glass to calculate the Rws ratio

        Parameters
        ==========
        self: object
            a rameau object that has been initiated.
        method: string
            The used method. LL2012: Le Losq et al. (2012); DG2017: Di Genova et al. (2017). See references. Default = "LL2012".
        delim: string
            File delimiter. Use '\t' for tabulated text or ',' for comma separated text. Default = '\t'.
        path_in: string
            Path for the spectra. Default = './raw/'
        laser: float
            Laser line wavelength in nm. Default = 514.532.
        spline_coeff: float
            Smoothing coefficient for the spline baseline. An array of size len(data_liste) can be provided. Default = 0.001.
        poly_coeff: int
            Polynomial coefficient for the polynomial baseline function. Default = 3 (DG2017 method; set to 2 for Behrens et al. (2006) method).

        Returns
        =======
        self.x: ndarray
            Common x axis.
        self.y_all: ndarray
            All raw spectra from data_liste in an array of length len(x) and with as many column as spectra.
        self.y_all_corr: ndarray
            All corrected spectra from data_liste in an array of length len(x) and with as many column as spectra.
        self.y_all_base: ndarray
            All baselines for spectra from data_liste in an array of length len(x) and with as many column as spectra.
        self.rws: ndarray
            The ratio of the water integrated intensity over that of silicate signals.
        self.rw: ndarray
            The integrated intensity of water signal.
        self.rs: ndarray
            The integrated intensity of silicate signals.
        """
        self.x, self.y, self.y_corr, self.y_base, self.rws, self.rw, self.rs = fit_spectra(self.data_liste,method=method,delim=delim,path_in=path_in,spline_coeff=spline_coeff,poly_coeff=poly_coeff)

    def calibrate(self,method="LL2012"):
        """Fit a calibration by optimizing the K coefficient(s)

        Parameters
        ----------
        self: object
            rameau object with treated spectra (see data_reduction method)
        method: string
            the method used; choose between "LL2012" or "DG2017", default = "LL2012".

        Returns
        -------
        popt: ndarray or float
            The optimized parameter(s);
            if method = "DG2017", popt=np.array([a,b]), parameters of the equation K = a * [FeO wt%] + b.
            if method = "LL2017", popt = A (float), with A parameter in the equation (3) of Le Losq et al. (2012).
        """
        dictio = {"water": self.water,
         "feo": np.asarray(self.data_liste["FeO"]),
         "rws": self.rws}
        try:
            if method == "LL2012":
                self.p = LL2012_calibrate(dictio)
            elif method == "DG2017":
                self.p = DG2017_calibrate(dictio)
        except:
            raise TypeError('Set method: Choose between LL2012 and DG2017 methods.')

    def predict(self,method="LL2012"):
        """predict the water content from the Rws

        Parameters
        ----------
        self: object
            rameau object with treated spectra (see data_reduction method).
        method: string
            the method used; choose between "LL2012" or "DG2017", default = "LL2012".

        Returns
        -------
        H2O
            The glass water contents in wt%
        """
        dictio = {"feo": np.asarray(self.data_liste["FeO"]),
         "rws": self.rws}

        if method == "LL2012":
            try:
                if self.p.size > 0:
                    print("Using adjusted A coefficient: %f" % self.p)
                    self.water_predicted = LL2012_predict(dictio,A=self.p)
                else:
                    print("Using the A coefficient = 0.007609 from Le Losq et al. 2012...")
                    self.water_predicted = LL2012_predict(dictio)
            except:
                raise TypeError("Bad p coefficient. Did you try predicting values with a different method than the calibration?")
        elif method == "DG2017":
            try:
                if self.p.size > 0:
                    print("Using adjusted p coefficients: %f and %f" % (self.p[0],self.p[1]))
                    self.water_predicted = DG2017_predict(dictio,a=self.p[0], b=self.p[1])
                else:
                    print("Using the p coefficients 0.096 and 0.663 from Di Genova et al. 2017...")
                    self.water_predicted = DG2017_predict(dictio)
            except:
                raise TypeError("Bad p coefficients. Did you try predicting values with a different method than the calibration?")

def fit_spectra(data_liste,method="LL2012",delim='\t',path_in='./raw/',laser=514.532,spline_coeff=0.001, poly_coeff=3):
    """Calculate the ratios of water and silicate signals from Raman spectra

    Parameters
    ----------
    data_liste: Pandas DataFrame
        Contains the list of spectra, see provided file as an example
    method: string
        The used method. LL2012: Le Losq et al. (2012); DG2017: Di Genova et al. (2017). See references.
    delim: string
        File delimiter. Use '\t' for tabulated text or ',' for comma separated text.
    path_in: string
        Path for the spectra
    laser: float
        Laser line wavelength in nm
    spline_coeff: float
        Smoothing coefficient for the spline baseline. An array of size len(data_liste) can be provided. Default = 0.001.
    poly_coeff: int
        Polynomial coefficient for the polynomial baseline function. Default = 3 (DG2017 method). Set to 2 for Behrens et al. (2006) method.

    Returns
    -------
    x: ndarray
        Common x axis.
    y_all: ndarray
        All raw spectra from data_liste in an array of length len(x) and with as many column as spectra.
    y_all_corr: ndarray
        All corrected spectra from data_liste in an array of length len(x) and with as many column as spectra.
    y_all_base: ndarray
        All baselines for spectra from data_liste in an array of length len(x) and with as many column as spectra.
    rws: ndarray
        The ratio of the water integrated intensity over that of silicate signals.
    rw: ndarray
        The integrated intensity of water signal.
    rs: ndarray
        The integrated intensity of silicate signals.

    Raises
    ------
    IOError
        If method is not set to LL2012 or DG2017.

    References
    ----------
    C. Le Losq, D. R. Neuville, R. Moretti, J. Roux, Determination of water content in silicate glasses using Raman spectrometry: Implications for the study of explosive volcanism. American Mineralogist. 97, 779–790 (2012).
    D. Di Genova et al., Effect of iron and nanolites on Raman spectra of volcanic glasses: A reassessment of existing strategies to estimate the water content. Chemical Geology. 475, 76–86 (2017).
    """

    import gcvspline
    
    x_all_lf = np.arange(50,1400,1.0)
    x_all_hf = np.arange(2800,3800,1.0)
    x = np.hstack((x_all_lf,x_all_hf))
    y_all = np.zeros((len(x),len(data_liste)))
    y_all_base = np.copy(y_all)
    y_all_corr = np.copy(y_all)

    rws = np.ones(len(data_liste))
    rw = np.ones(len(data_liste))
    rs = np.ones(len(data_liste))

    record_std = np.zeros((len(data_liste),2))

    rois = data_liste.loc[:,"ROI1 lb":"ROI6 hb"]

    for i in range(len(data_liste)):

        # importing the spectra
        sp = np.genfromtxt("./raw/"+data_liste["Name"][i],delimiter=delim,skip_header=1)

        # constructing an interpolator: this will allow an output of all data with the same X axis
        f = scipy.interpolate.interp1d(sp[:,0], sp[:,1],fill_value="extrapolate")

        # temperature and excitation line correction (see Rameau help)
        x, y_all[:,i], sdf = rp.tlcorrection(x,f(x),23.0,laser,normalisation='intensity')

        # getting the roi
        roi = np.array(rois.loc[i]).reshape(int(len(rois.loc[i])/2),2)

        # calculating baseline
        if method == "LL2012": # spline
            
            try:
                c_hf, b_hf = rp.baseline(x,y_all[:,i],roi,"gcvspline",s=spline_coeff)
            except:
                break

            y_all_corr[:,i]=c_hf[:,0]
            y_all_base[:,i]=b_hf[:,0]

        elif method == "DG2017": # polynomial 3 following DG2017 method

            # getting portion of interrest
            x_lf = x[np.where(x<2000.)].reshape(-1)
            x_hf = x[np.where(x>2000.)].reshape(-1)

            y_lf = y_all[np.where(x<2000.),i].reshape(-1)
            y_hf = y_all[np.where(x>2000.),i].reshape(-1)

            c_lf, b_lf = rp.baseline(x_lf,y_lf,np.array([[0,200],[1240,1500]]),"poly",polynomial_order = poly_coeff)
            c_hf, b_hf = rp.baseline(x_hf,y_hf,np.array([[2500,3100],[3750,3900]]),"poly",polynomial_order = poly_coeff)

            y_all_corr[:,i] = np.hstack((c_lf.reshape(-1),c_hf.reshape(-1)))
            y_all_base[:,i] = np.hstack((b_lf.reshape(-1),b_hf.reshape(-1)))

        else:
            raise TypeError('method should be set to LL2012 or DG2017')

        # Area / Integrated Intensity calculation
        S = np.trapz(y_all_corr[np.where((x>150)&(x<1250)),i],x[np.where((x>150)&(x<1250))])
        W = np.trapz(y_all_corr[np.where((x>3100)&(x<3750)),i],x[np.where((x>3100)&(x<3750))])

        # updating the Pandas dataframe rws
        rs[i] = S[0]
        rw[i] = W[0]
        rws[i] = W[0]/S[0]

    return x, y_all, y_all_corr, y_all_base, rws, rw, rs

def DG2017_predict(dictio,a=0.096,b=0.663):
    """Calculate the K coefficient for the DG2017 method.

    Parameters
    ----------
    dictio: dict
        a dictionary with ndarrays named "feo" and "rws"
    a and b: float
        factors in the equation: K = a * [FeO wt%] + b; default values from Di Genova et al. (2017)

    Returns
    -------
    H2O (wt %): ndarray
        The water content of the glasses calculated as Rws * (a * [FeO wt%] + b)

    """
    return dictio["rws"]*(dictio["feo"]*a + b)

def DG2017_calibrate(dictio):
    """Fit a calibration by optimizing the K coefficient in the DG2017 method

    Parameters
    ----------
    dictio: dictionary
        dictionary with arrays named "feo", "rws" and "water".

    Returns
    -------
    popt: ndarray
        The optimize a and b parameters of the equation K = a * [FeO wt%] + b.
    """
    popt, pcov = scipy.optimize.curve_fit(DG2017_predict, dictio, dictio["water"])
    return popt

def LL2012_predict(dictio,A=0.007609):
    """Predict the water content using the equation (3) from Le Losq et al. (2012)

    Parameters
    ----------
    dictio: dict
        a dictionary with ndarray named "rws"

    Returns
    -------
    H2O
        The glass water contents in wt%
    """
    return 100*A*dictio["rws"]/(1 + A*dictio["rws"])

def LL2012_calibrate(dictio):
    """Fit a calibration line following equations (2) and (3) from Le Losq et al. (2012)

    Parameters
    ----------
    dictio
        dictionary with arrays named "feo", "rws" and "water".

    Returns
    -------
    A: float
        The parameter in the equation (3) of Le Losq et al. (2012).

    """
    A, pcov = scipy.optimize.curve_fit(LL2012_predict,dictio,dictio["water"])
    return A



#from numpy.distutils.core import setup
#from numpy.distutils.extension import Extension
from setuptools import setup, Extension

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='rampy',
      version='0.4.0',
      description='A Python module containing functions to treat spectroscopic (XANES, Raman, IR...) data',
      url='https://github.com/charlesll/rampy',
      author='Charles Le Losq',
      author_email='charles.lelosq@anu.edu.au',
      long_description=long_description,
      long_description_content_type="text/markdown",
      license='GNU-GPLv2',
      packages=['rampy', 'rampy.tests'],
      install_requires=['numpy>=1.12','scipy','sklearn','pandas','cvxpy>=1.0'],
      zip_safe=False)



#!/usr/bin/env python
#-*- coding: utf-8 -*-
import numpy as np
from scipy.special import erfc

############ SIMPLE MATHEMATICAL FUNCTIONS ###########
def gaussian(x,amp,freq,HWHM): # for spectral fit
    return amp*np.exp(-np.log(2)*((x-freq)/HWHM)**2)

def gaussianarea(Amplitude,HWHM,**options):
    """
    Return the area of a gaussian with inpu amplitude and HWHM.
    Options are eseAmplitude (None by default) and eseHWHM (None by default)
    """
    area = np.sqrt(np.pi/np.log(2))*Amplitude*HWHM
    if options.get("eseAmplitude") != None:
        eseAmplitude = options.get("eseAmplitude")
        if options.get("eseHWHM") != None:
            eseHWHM = options.get("eseHWHM")
            esearea = np.sqrt((np.pi/np.log(2)*HWHM)**2 * eseAmplitude**2 + (np.pi/np.log(2)*Amplitude)**2 * eseHWHM**2)
    else:
        esearea = None

    return area, esearea

def pseudovoigt(x,amp,freq,HWHM,LGratio): # for spectral fit
    return LGratio*(amp/(1+((x-freq)/HWHM)**2)) + (1-LGratio)*(amp*np.exp(-np.log(2)*((x-freq)/HWHM)**2))

def funlog(x,a,b,c,d):
    return a*np.log(-b*(x-c))-d*x**2

def funexp(x,a,b,c):
    return a*np.exp(b*(x-c))

def poly2(x,a,b,c):
    return a + b*x + c*x*x

def linear(x,a,b):
    return a + b*x

def linear0(x,a):
    return a*x

def constant(x,a):
    return np.zeros(len(x))+a


########### SPECIFIC GAUSSIAN/VOIGTR FUNCTIONS FOR USING WITH SCIPY OPTIMIZATION PROTOCOLS

def multigaussian(x,params):
    taille = len(params)
    y = np.zeros(len(x),4)
    for i in range(taille):
        y[:,taille+1] = params[taille,0]*np.exp(-np.log(2)*((x-params[taille,1])/params[taille,2])**2)
    y[:,0] = y[:,1]+y[:,2]+y[:,3]
    return y


def gauss_lsq(params,x):
    nbpic = int(len(params)/3)
    a = np.zeros((1,nbpic))
    b = np.zeros((1,nbpic))
    c = np.zeros((1,nbpic))
    y = np.zeros((len(x),nbpic))
    for n in range(nbpic):
        m = 2*n # little trick for correct indexation
        a[0,n] = params[n+m]
        b[0,n] = params[n+m+1]
        c[0,n] = params[n+m+2]
        y[:,n] = a[0,n]*np.exp(-np.log(2)*((x[:]-b[0,n])/c[0,n])**2)
    ytot = sum(y,1)

    return ytot

def gauss_lsq_lfix(params,x):
    nbpic = int((len(params)-2)/2)
    a = np.zeros((1,nbpic))
    b = np.zeros((1,nbpic))
    c = np.zeros((1,2)) # FWMH fixed and in first position of params
    c[0,0] = params[0]
    c[0,1] = params[1]
    b[0,:] = params[2:(nbpic+2)]
    a[0,:] = params[nbpic+2:(2*nbpic+2)]
    y = np.zeros((len(x),nbpic))
    for n in range(nbpic):
        if n == 0:
            y[:,n] = a[0,n]*np.exp(-np.log(2)*((x[:]-b[0,n])/c[0,0])**2)
        else:
            y[:,n] = a[0,n]*np.exp(-np.log(2)*((x[:]-b[0,n])/c[0,1])**2)
    ytot = sum(y,1)

    return ytot

########### SPECIFIC FUNCTIONS FOR CHEMICAL DIFFUSION


def diffshort(x, t, C0, C1, D):
    """
    Simple equation for the diffusion into a semi-infinite slab, see Crank 1975
    C0 is the concentration in the core
    C1 is the concentration at the border
    D is the diffusion coefficient in log10 unit, m^2.s^-1
    x is the profil length in meters
    t is the time in seconds
    """

    Cx = (C1 - C0) * erfc(x / (2. * np.sqrt((10**D)*t))) + C0

    return Cx

def difffull(x1, x2, t, C0, C1, D):
    """
    Simple equation for the diffusion into a semi-infinite slab, see Crank 1975
    C0 is the concentration in the core
    C1 is the concentration at the border
    D is the diffusion coefficient in log10 unit, m^2.s^-1
    x1 and x2 are the profil lengths from beginning and end respectively, in meters
    t is the time in seconds
    """
    x = (x2-x1)
    Cx = (C1 - C0) * ( erfc(x / (2. * np.sqrt((10**D)*t))) +  erfc((x) / (2. * np.sqrt((10**D)*t))))+ C0

    return Cx
